// ----------------------------------------------------------------------
// File: tetrisesque.inc
// Vers: 3.6
// Date: 
// Auth: H.A. Camp
//
// Desc: Tetrisesque Include File (contains all the macros) 
//
// ----------------------------------------------------------------------


// ----------------------------------------------------------------------
// Macros
// ----------------------------------------------------------------------


#macro ShowTGrid()
  #debug "----------------------------------------------------------\n"
  #declare X=XMin;
  #declare Z=ZMax;
  #while (Z >= ZMin)
    #debug concat("(", str(Z,3,0), ") ")
    #while (X <= XMax)
      #declare X2 = X-XMin+1;
      #declare Z2 = Z-ZMin+1;
      #if (strlen(Grid[X2][Z2])<2)
        #debug concat(" ", Grid[X2][Z2], " ")
      #else
        #debug concat(Grid[X2][Z2], " ")
      #end
      #declare X=X+1;
    #end
    #debug "\n"
    #declare X=XMin;
    #declare Z=Z-1;
  #end
  #debug "\n    ("
  #declare X=XMin;
  #while (X <= XMax)
    #debug concat(str(X,3,0))
    #declare X=X+1;
  #end
    #debug ")\n"
  #debug "----------------------------------------------------------\n"
#end


// ----------------------------------------------------------------------
// Check_Ground Macro - Checks to see if 'ground' (area) is clear for new block placement
// ----------------------------------------------------------------------

#macro Check_Ground(Start)
  #declare Norm = <0, 0, 0>; 
  #declare Inter = trace (Area, Start, -y, Norm); 
#end


// ----------------------------------------------------------------------
// Check_Ground Macro - Checks to see if 'ground' (area) is clear for new block placement
// ----------------------------------------------------------------------
#macro Try_Shape()
  
  #declare gridchk = 1;    // Check to make sure target grid location is "oo"
  #declare  defchk = 1;    // Check to make sure object[objnum][cnt] is defined - if not, stop
  #declare      ok = 1;    // Make sure nothing is already in cx,cz slot being checked.  If so, ok=0
  #local   maskcnt = 0;

  #while (ok & defchk)
    #declare mStart = vaxis_rotate(xmask[objnum][maskcnt],<0,1,0>,objrot*90); 
    #declare oStart = <X,10,Z>;
    #declare nStart = mStart + oStart;
    Check_Ground(nStart)
            
    #if (Inter.y != -0.1) 
      #declare ok = 0;
    #end
          
    #declare aX = dimension_size(Grid,1);
    #declare aZ = dimension_size(Grid,2);
    #declare gX = nStart.x-XMin+1; 
    #declare gZ = nStart.z-ZMin+1; 

    #if ((aX <= gX) | (aZ <= gZ)) 
      #declare ok=0;
    #else      
      #ifndef (Grid[gX][gZ])
        #declare ok=0;      
      #else
        #if(strcmp(Grid[gX][gZ],"oo")) 
          #declare ok=0; 
        #end
      #end
    #end
    #local maskcnt = maskcnt + 1;
    #ifndef (xmask[objnum][maskcnt]) #declare defchk = 0; #end
  #end
#end


// ----------------------------------------------------------------------
// Make_Block_Pattern Macro - The main block placement function
// ----------------------------------------------------------------------
#macro Place_Block()
  
  #declare Start = <X,10,Z>;  // First, check to make sure initial ground area is empty

  Check_Ground(Start)

  #if (Inter.y = -0.1)  // If it's empty, we're putting *something* in this slot
    #declare    Repeat =  0;
    #declare MaxRepeat = 20;
    
    #while (Repeat <= MaxRepeat)
      
      #declare objok = 1;
      #while (objok)
        #declare objnum = floor(rand(RSx)*0.9999999*(dimension_size(xobj,1)-1))+1;
        #declare objrot = floor(rand(RSx)*0.9999999*(4)); 
        #if ((objnum=13 & rand(RSx) >0.8) | (objnum != 13))
          #declare objok=0;
        #end
      #end

      Try_Shape() // If chosen block is allowed, ok=1

      #if (ok) 
        #declare Area = 
        union { 
          object {Area}
          #if (Z=-2 & X=0) 
            object {
              xobj[objnum] translate -0.5 rotate 90*objrot*y translate 0.5 scale <1,RRand(0.5, 1.5, RSx),1> translate <X-0.5,0,Z-0.5>   // Use this line if you want variable heights for each block 
              //xobj[objnum] translate -0.5 rotate 90*objrot*y translate 0.5 translate <X-0.5,0,Z-0.5>    // Use this line if you want all the blocks to be the same height
              texture {
                pigment {color rgb <0.8,0,0>}
                finish {
                  ambient 0.0
                  diffuse 0.86
                  brilliance 0.5
                  phong 0.95
                  phong_size 140
                  specular 0.15
                  roughness 0.0005
                }
              }
            }
          #else
            object {
              xobj[objnum] translate -0.5 rotate 90*objrot*y translate 0.5 scale <1,RRand(0.5, 1.5, RSx),1> translate <X-0.5,0,Z-0.5>   // Use this line if you want variable heights for each block 
              //xobj[objnum] translate -0.5 rotate 90*objrot*y translate 0.5 translate <X-0.5,0,Z-0.5>    // Use this line if you want all the blocks to be the same height
              texture {
                pigment {function {F0(X,-1,Z)*0.75+0.25}}
                finish {
                  ambient 0.0
                  diffuse 0.86
                  brilliance 0.5
                  phong 0.95
                  phong_size 140
                  specular 0.15
                  roughness 0.0005
                }
              }
            }
          #end
        }
        #declare blocknum = blocknum + 1;
        #declare Repeat = Repeat + 100;
      #else
        #if (Repeat >= MaxRepeat)
          #declare Area = union { object {Area} object { xobj[0] translate <X-0.5,0,Z-0.5> texture {pigment {function {F0(X,-1,Z)*0.75+0.25}}}}}
          #declare blocknum = blocknum + 1;
        #end
        #declare Repeat = Repeat + 1;
      #end // End ok check
    #end // End Repeat Loop
  #end // End Inter.y check
#end // End Place_Block() macro
